package com.streambase.contrib.stats;

import java.util.ArrayList;
import java.util.List;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.DataType;
import com.streambase.sb.client.CustomFunctionResolver;

public class Moments {
	
	/**
	 * 
	 * @param observations 	A list representing the sample data
	 * @param mean			The mean of the sample data
	 * @param variance		The variance of the sample data
	 * @return The skewness (the measure of the asymmetry of the data)
	 */
	@CustomFunctionResolver("MomentsCustomFunctionResolver")
	public static double Skewness(List<?> observations, double mean, double variance) {
		double sum = 0;
		int N = observations.size();
		for(int i = 0; i < N; i++){
			sum += Math.pow(mean - ((Double) observations.get(i)).doubleValue(), 3);
		}
		sum = sum/N;
		return sum/Math.pow(variance, 1.5);
	}
	
	/**
	 * 
	 * @param observations	A list representing the sample data
	 * @param mean			The mean of the sample data
	 * @param variance		The variance of the sample data
	 * @return The kurtosis (the "peakedness" of the data)
	 */
	@CustomFunctionResolver("MomentsCustomFunctionResolver")
	public static double Kurtosis(List<?> observations, double mean, double variance) {
		double sum = 0;
		int N = observations.size();
		for(int i = 0; i < N; i++){
			sum += Math.pow(mean - ((Double) observations.get(i)).doubleValue(), 4);
		}
		sum = sum/N;
		return sum/Math.pow(variance, 2);
	}
	
	/**
	 * 
	 * @param observations 	A list representing the sample data
	 * @param mean			The mean of the sample data
	 * @param variance		The variance of the sample data
	 * @return A list representing the standardized observations
	 */
	@CustomFunctionResolver("StandardizedCustomFunctionResolver")
	public static List<?> Standardize(List<?> observations, double mean, double variance) {
		int N = observations.size();
		ArrayList<Double> standardizedData = new ArrayList<Double>();
		for(int i = 0; i < N; i++){
			standardizedData.add((mean - (Double) observations.get(i))/Math.sqrt(variance));
		}
		return standardizedData;
	}
	
	/**
	 * 
	 * @param observations 	A list representing the sample data
	 * @param NameOfTest	The R-function name of the normality test
	 * @return A string representing the r-function call
	 */
	@CustomFunctionResolver("RCustomFunctionResolver")
	public static String CreateRExpression(List<?> observations, String NameOfTest, boolean addArgs) {		
		// Get the string version of the list
		String lis = observations.toString();		
		
		// Convert the list brackets in java to list brackets in R
		String result = "c(".concat(lis.substring(1,lis.length()-1)).concat(")");
		
		// Some tests have suffix arguments that we need to add
		if (addArgs)
			return  String.format("%s(%s, \"pnorm\")$p.value",NameOfTest,result);
		return  String.format("%s(%s)$p.value",NameOfTest,result);
	}
	
	/**
	 * 
	 * @param quantile	The chi-square quantile value
	 * @return A string representing the r-function call
	 */
	public static String ChiSquare(double quantile){		
		return  String.format("1-pchisq(%f,df=2)",quantile);
	}
	

// ************************************************************************************************/
// ****************************** FUNCTION RESOLVERS **********************************************/
// ************************************************************************************************/
	
	public static CompleteDataType MomentsCustomFunctionResolver(
			CompleteDataType observations, CompleteDataType mean,
			CompleteDataType variance) {
		
		CompleteDataType element = observations.getElementType();	
		if (element == null || element.getDataType() != DataType.DOUBLE) {return null;}
		if (mean.getDataType() != DataType.DOUBLE || variance.getDataType() != DataType.DOUBLE) {return null;}		
		return CompleteDataType.forDouble();
	}
	
	public static CompleteDataType StandardizedCustomFunctionResolver(
			CompleteDataType observations, CompleteDataType mean,
			CompleteDataType variance) {
		
		CompleteDataType element = observations.getElementType();	
		if (element == null || element.getDataType() != DataType.DOUBLE) {return null;}
		if (mean.getDataType() != DataType.DOUBLE || variance.getDataType() != DataType.DOUBLE) {return null;}		
		return CompleteDataType.forList(CompleteDataType.forDouble());
	}
	
	public static CompleteDataType RCustomFunctionResolver(
			CompleteDataType observations, CompleteDataType NameOfFunction, CompleteDataType addArgs) {
		
		CompleteDataType element = observations.getElementType();	
		if (element == null) {return null;}
		if (NameOfFunction.getDataType() != DataType.STRING) {return null;}
		if (addArgs.getDataType() != DataType.BOOL) {return null;}
		if (element.getDataType() == DataType.DOUBLE) {return CompleteDataType.forString();}
		if (element.getDataType() == DataType.INT) {return CompleteDataType.forString();}
		if (element.getDataType() == DataType.LONG) {return CompleteDataType.forString();}
		return null;
	}
	
}
